# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, TYPE_CHECKING

from azure.core.pipeline import policies
from azure.core.rest import AsyncHttpResponse, HttpRequest
from azure.mgmt.core import AsyncARMPipelineClient
from azure.mgmt.core.policies import AsyncARMAutoResourceProviderRegistrationPolicy

from .. import models as _models
from .._serialization import Deserializer, Serializer
from ._configuration import OracleDatabaseMgmtClientConfiguration
from .operations import (
    AutonomousDatabaseBackupsOperations,
    AutonomousDatabaseCharacterSetsOperations,
    AutonomousDatabaseNationalCharacterSetsOperations,
    AutonomousDatabaseVersionsOperations,
    AutonomousDatabasesOperations,
    CloudExadataInfrastructuresOperations,
    CloudVmClustersOperations,
    DbNodesOperations,
    DbServersOperations,
    DbSystemShapesOperations,
    DnsPrivateViewsOperations,
    DnsPrivateZonesOperations,
    GiVersionsOperations,
    Operations,
    OracleSubscriptionsOperations,
    SystemVersionsOperations,
    VirtualNetworkAddressesOperations,
)

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential


class OracleDatabaseMgmtClient:  # pylint: disable=client-accepts-api-version-keyword,too-many-instance-attributes
    """OracleDatabaseMgmtClient.

    :ivar operations: Operations operations
    :vartype operations: azure.mgmt.oracledatabase.aio.operations.Operations
    :ivar autonomous_databases: AutonomousDatabasesOperations operations
    :vartype autonomous_databases:
     azure.mgmt.oracledatabase.aio.operations.AutonomousDatabasesOperations
    :ivar cloud_exadata_infrastructures: CloudExadataInfrastructuresOperations operations
    :vartype cloud_exadata_infrastructures:
     azure.mgmt.oracledatabase.aio.operations.CloudExadataInfrastructuresOperations
    :ivar cloud_vm_clusters: CloudVmClustersOperations operations
    :vartype cloud_vm_clusters: azure.mgmt.oracledatabase.aio.operations.CloudVmClustersOperations
    :ivar autonomous_database_character_sets: AutonomousDatabaseCharacterSetsOperations operations
    :vartype autonomous_database_character_sets:
     azure.mgmt.oracledatabase.aio.operations.AutonomousDatabaseCharacterSetsOperations
    :ivar autonomous_database_national_character_sets:
     AutonomousDatabaseNationalCharacterSetsOperations operations
    :vartype autonomous_database_national_character_sets:
     azure.mgmt.oracledatabase.aio.operations.AutonomousDatabaseNationalCharacterSetsOperations
    :ivar autonomous_database_versions: AutonomousDatabaseVersionsOperations operations
    :vartype autonomous_database_versions:
     azure.mgmt.oracledatabase.aio.operations.AutonomousDatabaseVersionsOperations
    :ivar db_system_shapes: DbSystemShapesOperations operations
    :vartype db_system_shapes: azure.mgmt.oracledatabase.aio.operations.DbSystemShapesOperations
    :ivar dns_private_views: DnsPrivateViewsOperations operations
    :vartype dns_private_views: azure.mgmt.oracledatabase.aio.operations.DnsPrivateViewsOperations
    :ivar dns_private_zones: DnsPrivateZonesOperations operations
    :vartype dns_private_zones: azure.mgmt.oracledatabase.aio.operations.DnsPrivateZonesOperations
    :ivar gi_versions: GiVersionsOperations operations
    :vartype gi_versions: azure.mgmt.oracledatabase.aio.operations.GiVersionsOperations
    :ivar system_versions: SystemVersionsOperations operations
    :vartype system_versions: azure.mgmt.oracledatabase.aio.operations.SystemVersionsOperations
    :ivar oracle_subscriptions: OracleSubscriptionsOperations operations
    :vartype oracle_subscriptions:
     azure.mgmt.oracledatabase.aio.operations.OracleSubscriptionsOperations
    :ivar autonomous_database_backups: AutonomousDatabaseBackupsOperations operations
    :vartype autonomous_database_backups:
     azure.mgmt.oracledatabase.aio.operations.AutonomousDatabaseBackupsOperations
    :ivar db_servers: DbServersOperations operations
    :vartype db_servers: azure.mgmt.oracledatabase.aio.operations.DbServersOperations
    :ivar db_nodes: DbNodesOperations operations
    :vartype db_nodes: azure.mgmt.oracledatabase.aio.operations.DbNodesOperations
    :ivar virtual_network_addresses: VirtualNetworkAddressesOperations operations
    :vartype virtual_network_addresses:
     azure.mgmt.oracledatabase.aio.operations.VirtualNetworkAddressesOperations
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :param subscription_id: The ID of the target subscription. The value must be an UUID. Required.
    :type subscription_id: str
    :param base_url: Service URL. Default value is "https://management.azure.com".
    :type base_url: str
    :keyword api_version: Api Version. Default value is "2023-09-01". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
     Retry-After header is present.
    """

    def __init__(
        self,
        credential: "AsyncTokenCredential",
        subscription_id: str,
        base_url: str = "https://management.azure.com",
        **kwargs: Any
    ) -> None:
        self._config = OracleDatabaseMgmtClientConfiguration(
            credential=credential, subscription_id=subscription_id, **kwargs
        )
        _policies = kwargs.pop("policies", None)
        if _policies is None:
            _policies = [
                policies.RequestIdPolicy(**kwargs),
                self._config.headers_policy,
                self._config.user_agent_policy,
                self._config.proxy_policy,
                policies.ContentDecodePolicy(**kwargs),
                AsyncARMAutoResourceProviderRegistrationPolicy(),
                self._config.redirect_policy,
                self._config.retry_policy,
                self._config.authentication_policy,
                self._config.custom_hook_policy,
                self._config.logging_policy,
                policies.DistributedTracingPolicy(**kwargs),
                policies.SensitiveHeaderCleanupPolicy(**kwargs) if self._config.redirect_policy else None,
                self._config.http_logging_policy,
            ]
        self._client: AsyncARMPipelineClient = AsyncARMPipelineClient(base_url=base_url, policies=_policies, **kwargs)

        client_models = {k: v for k, v in _models.__dict__.items() if isinstance(v, type)}
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.operations = Operations(self._client, self._config, self._serialize, self._deserialize)
        self.autonomous_databases = AutonomousDatabasesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.cloud_exadata_infrastructures = CloudExadataInfrastructuresOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.cloud_vm_clusters = CloudVmClustersOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.autonomous_database_character_sets = AutonomousDatabaseCharacterSetsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.autonomous_database_national_character_sets = AutonomousDatabaseNationalCharacterSetsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.autonomous_database_versions = AutonomousDatabaseVersionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.db_system_shapes = DbSystemShapesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.dns_private_views = DnsPrivateViewsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.dns_private_zones = DnsPrivateZonesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.gi_versions = GiVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.system_versions = SystemVersionsOperations(self._client, self._config, self._serialize, self._deserialize)
        self.oracle_subscriptions = OracleSubscriptionsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.autonomous_database_backups = AutonomousDatabaseBackupsOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.db_servers = DbServersOperations(self._client, self._config, self._serialize, self._deserialize)
        self.db_nodes = DbNodesOperations(self._client, self._config, self._serialize, self._deserialize)
        self.virtual_network_addresses = VirtualNetworkAddressesOperations(
            self._client, self._config, self._serialize, self._deserialize
        )

    def _send_request(
        self, request: HttpRequest, *, stream: bool = False, **kwargs: Any
    ) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client._send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        request_copy.url = self._client.format_url(request_copy.url)
        return self._client.send_request(request_copy, stream=stream, **kwargs)  # type: ignore

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> "OracleDatabaseMgmtClient":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)
